import types
from types import TupleType, ListType, IntType, SliceType

class ArrayFactory:
	def __getitem__(self,dims):
		ds=[]
		if not isinstance(dims,TupleType):
			dims=(dims,)
		for d in dims:
			if isinstance(d,IntType):
				ds.append((0,d))
			elif isinstance(d,SliceType):
				if d.step!=None and d.step!=1:
					raise ValueError( \
						'step in slice not '\
						'implemented')
				L=d.start
				U=d.stop
				if L==None or U==None:
				    raise valueError( \
					'bounds must be specified ' \
					'when creating an array')
				if not (isinstance(L,IntType) and \
						isinstance(U,IntType)):
				    raise valueError( \
					'bounds must be specified ' \
					'when creating an array')
				ds.append((L,U))
			else:
				raise valueError( \
					'bounds must be int or int:int')
		return Array(ds)
makeArray=ArrayFactory()
class Array:
	def __init__(self,*dims):
		if len(dims)==1 and (isinstance(dims[0],TupleType) \
				or isinstance(dims[0],ListType)):
			dims=dims[0]
		if len(dims)==0: 
			self._strides=()
			self._bounds=self._strides
			self._body=self._strides
			self._offset=0
			self._LBD=self._strides
			self._UBD=self._strides
			return
		ds=[]
		for d in dims:
			if isinstance(d,TupleType):
				if not len(d)==2: 
					raise ValueError( \
						'bad dimension tuple')
				try:
					ds.append((int(d[0]),int(d[1])))
				except:
					raise ValueError('non int bounds')
			else:
				try:
					d=int(d)
					ds.append((0,d))
				except:
					raise ValueError('non int bound')
		for d in ds:
			if d[0]>=d[1]:
				raise ValueError( \
					'upper bound<=lower bound',d)
		stride=1
		strides=[]
		for i in xrange(len(ds)-1,-1,-1):
			strides.insert(0,stride)
			stride *=( ds[i][1]-ds[i][0] )
		off=0
		for i in xrange(len(ds)):
			off -= ds[i][0]*strides[i]
		self._strides=tuple(strides)
		self._bounds=tuple(ds)
		self._body=[0]*stride
		self._offset=off
		self._LBD=tuple(map(lambda d:d[0], ds))
		self._UBD=tuple(map(lambda d:d[1], ds))
	def _clean_subs(self,x):
		slices=0
		if not isinstance(x,TupleType):
			x=(x,)
		if len(x)!=len(self._bounds): 
			raise ValueError( \
				'requires %d subscripts, got %d' % \
				(len(self._bounds),len(x)) )
		for i in xrange(len(self._bounds)):
			if isinstance(x[i],SliceType):
				slices=1
			elif not self._LBD[i]<=x[i]<self._UBD[i]:
				raise ValueError(\
				  'subscript[%d]=%d not in range [%d,%d)' \
				   % (i,x[i],self._LBD[i],self._UBD[i]) )
		return x,slices
	def _slice_trim(self,x):
		ubd=[]
		lbd=[]
		strides=[]
		off=self._offset
		for i in xrange(len(self._LBD)):
			d=x[i]
			if isinstance(d,SliceType):
				if d.step!=None:
					raise NotImplementedError( \
					  'step in slice not implemented')
				L=d.start
				U=d.stop
				if L==None: L=self._LBD[i]
				if U==None: U=self._UBD[i]
				lbd.append(max(L,self._LBD[i]))
				ubd.append(min(U,self._UBD[i]))
				strides.append(self._strides[i])
			else:
				d=int(d)
				if not self._LBD[i]<=d<self._UBD[i]:
				    raise ValueError(\
					'subscript[%d]=%d not in range [%d,%d)'\
					 % (i,d,self._LBD[i],self._UBD[i]) )
				off+=d*self._strides[i]
		a=Array()
		a._strides=tuple(strides)
		a._bounds=zip(lbd,ubd)
		a._body=self._body
		a._offset=off
		a._LBD=tuple(lbd)
		a._UBD=tuple(ubd)
		return a
		
	def __getitem__(self,x):
		x,slices=self._clean_subs(x)
		off = self._offset
		if slices:
			return self._slice_trim(x)
		for i in xrange(len(self._bounds)):
			off+=x[i]*self._strides[i]
		return self._body[off]
	def __setitem__(self,x,y):
		x,slices=self._clean_subs(x)
		if slices:
			dst=self._slice_trim(x)
			if not isinstance(y,Array):
				dst.fill(y)
				return
			else:
				_check_identical_shapes(dst,y)
				_visit_pairs(dst,y,lambda x,y:y)
				return
		off = self._offset
		for i in xrange(len(self._bounds)):
			off+=x[i]*self._strides[i]
		self._body[off]=y
	def LB(self,i): 
		return self._LBD[i]
	def UB(self,i): 
		return self._UBD[i]
	def bounds(self):
		return self._bounds
	def ndims(self): 
		return len(self._strides)
	def fill(self,val):
		_visit(self,lambda x,val=val:val)
		return self
	def unravel(self):
		class buildList:
			def __init__(self):
				self.L=[]
			def __call__(self,x):
				self.L.append(x)
				return x
		x=buildList()
		_visit(self,x)
		return x.L
	def ravel(self,L):
		class deliverList:
			def __init__(self,L):
				self.L=list(L)
				self.L.reverse()
			def __call__(self,x):
				return self.L.pop()
		x=deliverList(L)
		_visit(self,x)
		return self
def _check_identical_shapes(x,y):
	if x.ndims()!=y.ndims():
		raise ValueError( \
			'array assignment: not identical shapes')
	for i in xrange(x.ndims()):
		if (x.UB(i)-x.LB(i)) != (y.UB(i)-y.LB(i)):
			raise ValueError( \
				'array assignment: not identical shapes')
def _visit(dst,v):
	_visit_off(0,dst,dst._offset,v)

def _visit_off(i,dst,doff,v):
	dststride = dst._strides[i]
	d=dst._LBD[i]*dststride
	if i==dst.ndims()-1:
		for k in xrange(dst._LBD[i],dst._UBD[i]):
			dst._body[doff+d]=v(dst._body[doff+d])
			d+=dststride
	else:
		for k in xrange(dst._LBD[i],dst._UBD[i]):
			_visit_off(i+1,dst,doff+d,v)
			d+=dststride
	return

def _visit_pairs(dst,src,v):
	_visit_pairs_off(0,dst,dst._offset,src,src._offset,v)

def _visit_pairs_off(i,dst,doff,src,soff,v):
	srcstride=src._strides[i]
	s=src._LBD[i]*srcstride
	dststride=dst._strides[i]
	d=dst._LBD[i]*dststride
	if i==dst.ndims()-1:
		for k in xrange(dst._LBD[i],dst._UBD[i]):
			dst._body[doff+d] = \
				v(dst._body[doff+d],src._body[soff+s])
			s+=srcstride
			d+=dststride
	else:
		for k in xrange(src._LBD[i],src._UBD[i]):
			_visit_pairs_off(i+1,dst,doff+d,src,soff+s,v)
			s+=srcstride
			d+=dststride


