""" Multi map """
import Set
import PureSet
emptySet=PureSet.emptySet

class mmap:
	"""mmap(stuff=())
	Creates a multimap, a set of ordered pairs. It can
	be viewed either as a set of pairs (x,y) or as a 
	dictionary mapping keys x into sets of values {y,...}. 
	If present, stuff is a sequence or set of pairs.
	"""
	def __init__(self,stuff=()):
		self.rep={}
		if isinstance(stuff,Set.Set):
			stuff=stuff.items()
		for x in stuff:
			self[x[0]]=x[1]
	def insert(self,*x):
		"""m.insert(x[,y])
		put the pair x or (x,y) into the mmap
		if y is omitted, x must be a sequence of 
		length 2 whose elements are then inserted"""
		if len(x)==1: x=x[0]
		self[x[0]]=x[1]
	def delete(self,*x):
		"""m.delete(x[,y])
		removes the pair (x,y) from the mmap
		if y is omitted, x must be a sequence of 
		length 2 whose elements are the pair deleted"""
		if len(x)==1: x=x[0]
		k=x[0]
		s=self.rep.get(k)
		if s: 
			s.delete(x[1])
			if len(s)==0:
				del self.rep[k]
	def update(self,other):
		"""update(self,other)
		inserts all pairs in other into self
		other can be a multimap or a set or
		sequence of pairs
		"""
		if isinstance(other,mmap) or \
			isinstance(other,Set.Set):
				other=other.items()
		for x in other:
			self.insert(x)
	def with(self,*i):
		"""with(self,x[,y])
		returns a copy of self with the pair x or (x,y) inserted
		"""
		if len(i)==1: i=i[0]
		t=self.copy()
		t.insert(i)
		return t
	def without(self,*i):
		"""without(self,x[,y])
		returns a copy of self with the pair x or (x,y) removed
		"""
		if len(i)==1: i=i[0]
		t=self.copy()
		t.delete(i)
		return t
	def contains(self,*x):
		"""s.contains(x[,y])
		returns true if s contains pair x or (x,y)
		"""
		if len(x)==1: x=x[0]
		return self.rep.has_key(x[0]) and \
			self.rep[x[0]].contains(x[1])
	def has_key(self,k):
		"""has_key(self,k)
		returns true if self contains pair (k,y) for some y
		"""
		return self.rep.has_key(k)
	def items(self):
		"""items(self)
		returns a list of all the pairs in self
		"""
		L=[]
		for k in self.rep.keys():
			for v in self.rep[k].members():
				L.append((k,v))
		return L
	def keys(self):
		"""keys(self)
		returns a list [x,...] containing all values x
		for which self contains a pair (x,y) for some y
		"""
		return self.rep.keys()
	def values(self):
		"""values(self)
		returns a list [y,...] containing all values y
		for which self contains a pair (x,y) for some x
		"""
		L=[]
		for s in self.rep.values():
			L=L+s.items()
		return L
	def get(self,k,f=None):
		"""get(self,k,f=None)
		returns a pure set of all values y s.t.
		(k,y) is in self. If there are no such
		values y, it returns f.
		"""
		if self.rep.has_key(k):
			return PureSet.PureSet(rep=self.rep[k])
		else:	return f
	def members(self):
		"""members(self)
		returns a list of all the members in self
		equivalent to self.items() 
		"""
		return self.items()
	def clear(self):
		"""clear(self)
		removes all pairs from self
		"""
		self.rep={}
	def copy(self):
		"""s.copy()
		returns a copy of the multimap s
		"""
		x=mmap()
		x.rep={}
		for k in self.rep.keys():
			x.rep[k]=self.rep[k].copy()
		return x
	def new(self):
		"""new(self)
		creates a new, empty multimap
		"""
		x=mmap()
		return x
	def __or__(self,s):
		"""__or__(self,s), self|s
		returns a multimap whose elements are the
		union of self and s, i.e. all pairs (x,y)
		s.t. self.contains(x,y) or s.contains(x,y)
		"""
		return mmap(self.toSet()|s.toSet())
	def __and__(self,s):
		"""__and__(self,s), self&s
		returns a multimap whose elements are the
		intersection of self and s, i.e. all pairs (x,y)
		s.t. self.contains(x,y) and s.contains(x,y)
		"""
		x=self.new()
		for y in self.items():
			if s.contains(y):
				x.insert(y)
		return x
	def __sub__(self,s):
		"""__sub__(self,s), self-s
		returns a multimap whose elements are the
		relative complementof self and s, i.e. all 
		pairs (x,y) s.t. self.contains(x,y) and 
		not s.contains(x,y)
		"""
		x=self.new()
		for y in self.items():
			if not s.contains(y):
				x.insert(y)
		return x
	def __mul__(self,y):
		"""__mul__(x,y), x*y
		Composition. Returns the multimap z of
		all pairs (v,w) s.t. (v,t) is in x
		and (t,w) is in y for some t
		"""
		m=self.new()
		for k in self.keys():
		    for t in self[k].members():
			if y.has_key(t):
			    for v in y[t]:
				m[k]=v
		return m
	def toSet(self):
		"""toSet(self)
		returns the items in self as a set of pairs
		"""
		return Set.Set(self.items())
	def issubsetof(self,other):
		"""issubsetof(self,other)
		returns true if self considered as a set
		of pairs is a subset of other. Parameter
		other can either be a set or a multimap.
		"""
		if isinstance(other,mmap):
			other=other.toSet()
		return self.toSet().issubsetof(other)
	def ispropersubsetof(self,other):
		"""ispropersubsetof(self,other)
		returns true if self considered as a set
		of pairs is a proper subset of other. Parameter
		other can either be a set or a multimap.
		"""
		if isinstance(other,mmap):
			other=other.toSet()
		return self.toSet().ispropersubsetof(other)
	def equals(self,other):
		"""equals(self,other)
		returns true if self and other contain the same
		pairs
		"""
		if isinstance(other,mmap):
			other=other.toSet()
		return self.toSet().equals(other)
	def __cmp__(self,other):
		"""__cmp__(self,other)
		returns
			-1 if self is a proper subset of other
			0 if self == other
			1 otherwise
			Note: not symmetric """
		if isinstance(other,mmap):
			other=other.toSet()
		return self.toSet().__cmp__(other)
	def __getitem__(self,i):
		"""__getitem__(self,x), self[k]
		returns a set of all y s.t.
		self.contains(x,y)
		"""
		s=self.rep.get(i)
		if s: return PureSet.PureSet(rep=s)
		else: return PureSet.emptySet
	def __setitem__(self,k,v):
		"""__setitem__(self,k,v), self[k]=v
		inserts pair (k,v) into self
		"""
		s=self.rep.get(k)
		if not s: s=self.rep[k]=Set.Set()
		s.insert(v)
	def __delitem__(self,k):
		"""__delitem__(self,k), del self[k]
		removes all pairs (k,y) from self for any y
		"""
		del self.rep[k]
	def arb(self):
		"""arb(self)
		returns an arbitrary pair contained in self
		raises EmptySetException if there are no pairs 
		in self
		"""
		if self:
			x=self.rep.keys()[0]
			return (x,self.rep[x].arb())
		else:
			import SetExceptions
			raise SetExceptions.EmptySetException()
	def __len__(self):
		"""__len__(self), len(self)
		returns the number of pairs in self
		"""
		c=0
		for k in self.rep.keys():
			c=c+len(self.rep[k])
		return c
	def __nonzero__(self):
		"""__nonzero__(self), if self:...
		returns len(self)!=0
		"""
		return len(self.rep)!=0
	def __str__(self):
		"""__str__(self), str(self)
		returns a string representation of self taken as a set
		"""
		return str(self.toSet())
	def __repr__(self):
		"""__repr__(self)
		returns a call to mmap that will build a copy of self
		if all elements can be represented
		"""
		return "mmap(" + repr(self.items()) + ")"
