"""rational numbers"""
import types, math, hasher
def gcd(x,y):
	assert isinstance(x,types.IntType) or \
		isinstance(x,types.LongType)
	assert isinstance(y,types.IntType) or \
		isinstance(y,types.LongType)
	x=abs(x)
	y=abs(y)
	if y>x:
		x,y=y,x
	while y>0:
		x,y = y,x%y
	return x
class rational:
	def __init__(self,*args):
		if len(args)==0:
			self.numerator=0L
			self.denominator=1L
		elif len(args)==1:
			n=args[0]
			if isinstance(n,types.FloatType):
				f,x=math.frexp(n)
				self.numerator=long(f*2.0**64)
				self.denominator=2L**64
				if x<0: self.denominator*=2L**abs(x)
				else: self.numerator*=2L**x

			else:		
				self.numerator=long(n)
				self.denominator=1L
		else :
			self.numerator=long(args[0])
			self.denominator=long(args[1])
		g=gcd(self.numerator,self.denominator)
		if g<=1:return
		self.numerator=self.numerator/g
		self.denominator=self.denominator/g
	def __add__(x,y):
		numr=x.numerator*y.denominator+ \
			y.numerator*x.denominator
		denom=x.denominator*y.denominator
		g=gcd(numr,denom)
		return rational(numr/g,denom/g)
	def __sub__(x,y):
		numr=x.numerator*y.denominator- \
			y.numerator*x.denominator
		denom=x.denominator*y.denominator
		g=gcd(numr,denom)
		return rational(numr/g,denom/g)
	def __mul__(x,y):
		numr=x.numerator*y.numerator
		denom=x.denominator*y.denominator
		g=gcd(numr,denom)
		return rational(numr/g,denom/g)
	def __div__(x,y):
		numr=x.numerator*y.denominator
		denom=x.denominator*y.numerator
		g=gcd(numr,denom)
		return rational(numr/g,denom/g)
	def __radd__(y,x): return x+y
	def __rsub__(y,x): return x-y
	def __rmul__(y,x): return x*y
	def __rdiv__(y,x): return x/y
	def __neg__(x):
		return rational(-x.numerator,x.denominator)
	def __float__(self):
		return float(self.numerator)/float(self.denominator)
	def __long__(self):
		return self.numerator/self.denominator
	def __int__(self):
		return int(long(self))
	def __repr__(self):
		return "rational("+repr(self.numerator)+ \
			","+repr(self.denominator)+")"
	def __str__(self):
		return "("+str(self.numerator)+ \
			"/"+str(self.denominator)+")"
	def __cmp__(self,other):
		return cmp((self-other).numerator, 0L)
	def __hash__(self):
		x=hasher.hash1(hash(self.numerator))
		y=hasher.hash2(hash(self.denominator))
		return hasher.hash3(x-y)
	def __nonzero__(self):
		return self.numerator!=0
	def __coerce__(self,other):
		if isinstance(other,rational):
			return self,other
		return self,rational(other)

