class DisjointUnion:
	'du=DisjointUnion(): creates data structure for union/find'
	def __init__(self):
		self.count={}
		self.parent={}
	def union(self,x,y):
		'''du.union(x,y): merges the sets that x and y belong 
		to into a single set. As with find(), x and y do not 
		have to be known to the algorithm before the union.'''
		x=self.find(x)
		y=self.find(y)
		if x == y: return
		if self.count[x]>self.count[y]:
			x,y=y,x
		self.parent[x]=y
		self.count[y]=self.count[y]+self.count[x]
		del self.count[x]
	def find(self,x):
		'''du.find(x): finds the set that x is a member of. 
		The sets are indicated by a representative element. 
		If x has not been encountered before, it is inserted 
		into a set of its own and becomes its own representative element.'''
		if not self.count.has_key(x)     \
		   and not self.parent.has_key(x):
			self.count[x]=1
			return x
		y=x
		while self.parent.has_key(x):
			x=self.parent[x]
		while y != x:
			z=self.parent[y]
			self.parent[y]=x
			y=z
		return x
	def in_same_set(self,x,y):
		'''returns true if x and y are in the same set, 
		false otherwise. Again, x and y do not have to 
		be known before the test is performed, but they 
		will be known to the algorithm after the test.'''
		return self.find(x) == self.find(y)
	def clear(self): 
		'''resets the algorithm. No elements are known 
		to the algorithm afterwards.'''
		self.__init__()
		