#main program for simple calculator using TCLLk in Python
# includes semantics class

import toolsofcomputing_org.tcllk.LLkParser as LLkParser
import toolsofcomputing_org.tcllk.Token as Token
import toolsofcomputing_org.tcllk.ActionSemantics as ActionSemantics 
import toolsofcomputing_org.tcllk.ErrorOutput as ErrorOutput
import toolsofcomputing_org.tcllk.Discard as Discard
import toolsofcomputing_org.tcllk.PanicMode as PanicMode

import CalcScanner, Calc_Tables, sys
import operator

def isError(x):
	return isinstance(x,Token.ErrorToken)
	
def anyError(*L):
	for x in L:
		if isError(x):
			return x
	return None

def doBop(x,op,optoken,y):
	try:
		return op(x,y)
	except Exception, e:
		errorout.error(optoken, 'arithmetic error')
		return Token.mkErrorToken("num",str(e))
class CalcSemantics(ActionSemantics.ActionSemantics):
	def __init__(self):
		ActionSemantics.ActionSemantics.__init__(self)
		self.vars={}
	def STARTL(self): return None
	def EXPRSTMT(self,e): 
		if not isError(e):
			print e
	def STMT(self,s,semi): 
		Token.recycle(semi)
		return None
	def NULLSTMT(self,semi): 
		Token.recycle(semi)
		return None
	def ASGN(self,L,eq,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(eq)
		self.vars[L.body]=R
	def ADD(self,L,op,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(op)
		return doBop(L,operator.add,op,R)
	def SUB(self,L,op,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(op)
		return doBop(L,operator.sub,op,R)
	def MPY(self,L,op,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(op)
		return doBop(L,operator.mul,op,R)
	def DIV(self,L,op,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(op)
		return doBop(L,operator.div,op,R)
	def MOD(self,L,op,R): 
		e=anyError(L,R)
		if e: return e
		Token.recycle(op)
		return doBop(L,operator.mod,op,R)
	def CVTNUM(self,t):
		try: 
			v=eval(t.body)
		except:
			errorout.error(t, 'conversion error')
			v=Token.mkErrorToken("num",t.body)
		Token.recycle(t)
		return v
	def LOOKUPID(self,t):
		if isError(t): return t
		try:
		    try:
			return self.vars[t.body]
		    except:
			errorout.error(t, 'unknown variable')
			return Token.mkErrorToken(t.type,t.body)
		finally:
		    Token.recycle(t)

	def NEG(self,minus,e): 
		if isError(e): return e
		try:
		    try:
		    	return -e
		    except Exception, ex:
			errorout.error(minus,str(ex)) 
			return Token.mkErrorToken('num',str(ex))
		finally:
			Token.recycle(minus)
	def PARENS(self,Lpar,e,Rpar): 
		Token.recycle(Lpar,Rpar)
		return e

if len(sys.argv)>1:
	name=sys.argv[1]
	f=open(name)
else:
	name='stdin'
	f=sys.stdin
errorout=ErrorOutput.ErrorOutput()
p=LLkParser.LLkParser(CalcScanner.CalcScanner(f,name), \
	Calc_Tables,CalcSemantics(),errorout, \
	Discard.Discard("EOL",PanicMode.PanicMode()))

p.parse()
