#main program for simple calculator using TCLLk in Python
# includes semantics class

import toolsofcomputing_org.tcllk.LLkParser as LLkParser
import toolsofcomputing_org.tcllk.Token as Token
import toolsofcomputing_org.tcllk.ActionSemantics as ActionSemantics
import toolsofcomputing_org.tcllk.ErrorOutput as ErrorOutput

import CalcScanner, Calc_Tables, sys, operator

def isError(x):
	return isinstance(x,Token.ErrorToken)
	
def anyError(*L):
	for x in L:
		if isError(x):
			return x
	return None

class WriteOut:
	def __call__(self,node):
		getattr(self,node.__class__.__name__)(node)
	def Bop(self,node): print node.op,
	def Var(self,node): print node.name,
	def Neg(self,node): print '.-',
	def Number(self,node): print node.n,
	def Asgn(self,node): print node.id+'=',
	def ExprStmt(self,node): print "print",
	def Nothing(self,node): print 'Nothing',

class Nothing:
	def walk(self,f):
		f(self)
	def eval(self,d):
		return None

class Bop:
	opr={'+':operator.add,'-':operator.sub, \
		'*':operator.mul,'/':operator.div, \
		'%':operator.mod,';':lambda x,y:None}
	def __init__(self,x,op,y):
		self.left=x
		self.op=op
		self.right=y
	def walk(self,f):
		#print "Bop walk",self,self.left,self.right
		self.left.walk(f)
		self.right.walk(f)
		f(self)
	def eval(self,d):
		L=self.left.eval(d)
		R=self.right.eval(d)
		return self.opr[self.op](L,R)
		

def BOP(x,op,y):
	e=anyError(x,op,y)
	if e: return e
	return Bop(x,op.body,y)

class Var:
	def __init__(self,name):
		self.name=name.body
	def walk(self,f):
		f(self)
	def eval(self,d):
		return d[self.name]

class Neg:
	def __init__(self,minus,e):
		self.e=e
	def walk(self,f):
		self.e.walk(f)
		f(self)
	def eval(self,d):
		return - self.e.eval(d)
class Number:
	def __init__(self,n):
		self.n=n
	def walk(self,f):
		f(self)
	def eval(self,d):
		return self.n

class Asgn:
	def __init__(self,id,op,e):
		self.id=id.body
		self.e=e
	def walk(self,f):
		self.e.walk(f)
		f(self)
	def eval(self,d):
		d[self.id]=self.e.eval(d)
		return None
class ExprStmt:
	def __init__(self,e):
		self.e=e
	def eval(self,d):
		v=self.e.eval(d)
		print v
		return v
	def walk(self,f):
		self.e.walk(f)
		f(self)

class CalcSemantics(ActionSemantics.ActionSemantics):
	def __init__(self):
		ActionSemantics.ActionSemantics.__init__(self)
		self.ADD=BOP
		self.SUB=BOP
		self.MPY=BOP
		self.DIV=BOP
		self.MOD=BOP
		self.NEG=Neg
		self.STARTL=Nothing
	def STMT(self,sl,s,semi): 
		if isinstance(s,Token.ErrorToken):
			return sl
		return Bop(sl,';',s)
	def NULLSTMT(self,sl,semi): return sl
	def PARENS(self,Lpar,e,Rpar): 
		return e
	def ASGN(self,var,op,e):
		err=anyError(var,op,e)
		if err: return err
		return Asgn(var,op.body,e)
		
	def LOOKUPID(self,t):
		if isError(t): return t
		return Var(t)
	def CVTNUM(self,n):
		if isError(n): return n
		return Number(eval(n.body))
	def EXPRSTMT(self,e):
		if isError(e): return e
		return ExprStmt(e)

if __name__=='__main__':	
	if len(sys.argv)>1:
		name=sys.argv[1]
		f=open(name)
	else:
		name='stdin'
		f=sys.stdin
	csem=CalcSemantics()
	#print dir(csem)
	p=LLkParser.LLkParser(CalcScanner.CalcScanner(f,name),Calc_Tables, \
		csem,ErrorOutput.ErrorOutput())

	t=p.parse()

	#print "semantics:",t
	if t is not None: t.walk(WriteOut())
	print

	if t is not None: t.eval({})
