"SharedDB shared data base"

import threading

class SharedDBError:pass
class CommitError(SharedDBError):pass
class ClosedError(SharedDBError):pass
class InUseError(SharedDBError):pass

class SharedDB:
	def __init__(self,db):
		self.db=db
		self.time=0
		self.numOpen=0
			#num Transactions open
		self.amOpen=1
		self.stamp={}
			#key->timestamp
		self.numUsers={}
			#key->num Transactions using it
		self.lock=threading.RLock()
	def _timeStamp(self):
		if not self.amOpen: raise ClosedError
		self.time+=1
		return self.time
	def get(self,key):
	    self.lock.acquire()
	    try:
		if not self.amOpen: raise ClosedError
		v=self.db[key] #may raise exception
		if not self.stamp.has_key(key):
			ts=self.stamp[key]=self._timeStamp()
		else: ts=self.stamp[key]
		if self.numUsers.has_key(key):
			self.numUsers[key]+=1
		else: self.numUsers[key]=1
		return v,ts
	    finally:
		self.lock.release()
	def closeTransaction(self,stamps):
	    self.lock.acquire()
	    try:
		if not self.amOpen: raise ClosedError
		for k in stamps.keys():
			self.numUsers[k] -= 1  #moved from above
			if self.numUsers[k]==0:
				del self.numUsers[k]
				#if self.stamp.has_key(k): 
				#	del self.stamp[k]
				del self.stamp[k]
		self.numOpen-=1
	    finally:
		self.lock.release()
		
	def commit(self,stamps,changes):
	    self.lock.acquire()
	    try:
		if not self.amOpen: raise ClosedError
		error=0
		for k in stamps.keys():
			if stamps[k]<self.stamp[k]:
				error=1
				break
		if not error:
			for k in changes.keys():
				self.db[k]=changes[k]
				if self.stamp.has_key(k): 
					self.stamp[k]=self._timeStamp()
		self.closeTransaction(stamps)
		if error: raise CommitError
	    finally:
		self.lock.release()
	def open(self):
	    self.lock.acquire()
	    try:
		if not self.amOpen: raise ClosedError
		self.numOpen+=1
		return Transaction(self)
	    finally:
		self.lock.release()
	def close(self,force=0):
	    self.lock.acquire()
	    try:
		if not self.amOpen: raise ClosedError
		if self.numOpen>0 and not force: raise InUseError
		self.amOpen=0
		try: self.db.close()
		except: pass
	    finally:
		self.lock.release()

class Transaction:
	def __init__(self,shared):
		self.shared=shared
		self.values={}
		self.changed={}
		self.stamps={}
		self.amOpen=1
	def close(self,okay=1):
		if not self.amOpen: raise ClosedError
		self.amOpen=0
		if okay:
			self.shared.commit(self.stamps,self.changed)
		else:
			self.shared.closeTransaction(self.stamps)
			raise CommitError
	def __getitem__(self,key):
		if not self.amOpen: raise ClosedError
		if not self.values.has_key(key):
			self.values[key],self.stamps[key]= \
				self.shared.get(key)
		return self.values[key]
	def __setitem__(self,key,val):
		if not self.amOpen: raise ClosedError
		self.values[key]=self.changed[key]=val
	def abandon(self):
		self.shared.closeTransaction(self.stamps)

		
		



