''' Dining Philosophers problem synchronizing through
a SharedDB and running in a TransactionQueue, but
not using the TransactionQueue's facilities. '''
import threading
import toolsofcomputing_org.threads.SharedDB as SharedDB
import random
import toolsofcomputing_org.threads.TransactionQueue\
	 as TransactionQueue
import toolsofcomputing_org.threads.Future as Future

numPhilosophers=7
iterations=50
timer=threading.Condition()
forks=SharedDB.SharedDB({})
d=forks.open()
for i in range(numPhilosophers):
	d['fork%d' % i] = -1
d.close()
state=['.']*numPhilosophers

tq=TransactionQueue.TransactionQueue()

def showState():
	#print reduce(lambda x,y: x+y,state)
	print "".join(state)

def delay(t):
	timer.acquire()
	timer.wait(t)
	timer.release()


class Philosopher:
	def __init__(self,num):
		self.num=num
		self.leftFork='fork%d' % self.num
		self.rightFork='fork%d' % ((self.num+1)%numPhilosophers)
		self.iterations=iterations
		self.done=Future.Future()
	def getMemento(self): return None
	def setMemento(self,x): pass
	def run(self):
	    #print 'Philosopher',self.num,'entered'
	    try:
		leftFork=self.leftFork
		rightFork=self.rightFork
		#get forks
		while 1:
			d=forks.open()
			try:
				left=d[leftFork]
				right=d[rightFork]
				delay(random.random())
				if left == right == -1:
					d[leftFork]=d[rightFork]=self.num
					d.close()
					break
				d.close()
			except SharedDB.CommitError:
				#print '('+str(self.num)+")"
				delay(random.random())
				pass
		state[self.num]=str(self.num); showState()
		delay(1*random.random())
		d=forks.open()
		d[leftFork] = d[rightFork] = -1
		state[self.num]='.'; showState()
		d.close()
		#print 'Philosopher',self.num,'at bottom'
	    finally:
		#print 'Philosopher',self.num,'exited'
		self.iterations -= 1
		if self.iterations>0: tq.run(self)
		else: self.done.set(None)

diners=[]
def start():
	for i in range(numPhilosophers):
		p=Philosopher(i)
		diners.append(p)
		tq.run(p)

if __name__=='__main__':
	start()
	for p in diners:
		p.done.get()


