''' Dining Philosophers problem synchronizing through
a SharedDB and running in a TransactionQueue, and
using the TransactionQueue's facilities to restart
actions. '''
import threading
import toolsofcomputing_org.threads.SharedDB as SharedDB
import random
import toolsofcomputing_org.threads.TransactionQueue\
	 as TransactionQueue
import toolsofcomputing_org.threads.Future as Future

numPhilosophers=7
iterations=50
timer=threading.Condition()
forks=SharedDB.SharedDB({})
d=forks.open()
for i in range(numPhilosophers):
	d['fork%d' % i] = -1
d.close()
state=['.']*numPhilosophers

tq=TransactionQueue.TransactionQueue()

def showState():
	print "".join(state)

def delay(t):
	timer.acquire()
	timer.wait(t)
	timer.release()

class startEating:
	def __init__(self,num,leftFork,rightFork):
		self.num=num
		self.done=threading.Event()
		self.leftFork=leftFork
		self.rightFork=rightFork
	def getMemento(self): return None
	def setMemento(self,x): pass
	def run(self):
		leftFork=self.leftFork
		rightFork=self.rightFork
		d=forks.open()
		left=d[leftFork]
		right=d[rightFork]
		if left != -1 or right != -1:
			#print self.num,':',leftFork,left,rightFork,right
			d.close(0) #raises CommitError
		d[leftFork]=d[rightFork]=self.num
		delay(random.random())
		d.close()

		state[self.num]=str(self.num); showState()
		self.done.set()

class stopEating:
	def __init__(self,num,leftFork,rightFork):
		self.num=num
		self.done=threading.Event()
		self.leftFork=leftFork
		self.rightFork=rightFork
	def getMemento(self): return None
	def setMemento(self,x): pass
	def run(self):
		leftFork=self.leftFork
		rightFork=self.rightFork
		d=forks.open()
		d[leftFork] = d[rightFork] = -1
		delay(random.random())
		d.close()
		state[self.num]='.'; showState()
		self.done.set()

class Philosopher:
	def __init__(self,num):
		self.num=num
		self.leftFork='fork%d' % self.num
		self.rightFork='fork%d' % ((self.num+1)%numPhilosophers)
		self.iterations=iterations
		self.done=Future.Future()
	def getMemento(self): return None
	def setMemento(self,x): pass
	def run(self):
	  try:
	    for iternum in xrange(iterations):
		num=self.num
		leftFork=self.leftFork
		rightFork=self.rightFork
		cmd=startEating(num,leftFork,rightFork)
		tq.run(cmd)
		cmd.done.wait()

		delay(1*random.random())

		cmd=stopEating(num,leftFork,rightFork)
		tq.run(cmd)
		cmd.done.wait()

		delay(2*random.random())

	  finally:
		self.done.set(None)

diners=[]
def start():
	for i in range(numPhilosophers):
		p=Philosopher(i)
		diners.append(p)
		tq.run(p)

if __name__=='__main__':
	start()
	for p in diners:
		p.done.get()

